import { Plugin, PluginKey, NodeSelection } from 'prosemirror-state';
import { DecorationSet, Decoration } from 'prosemirror-view';
import { calculateBubblePos, isLinkActive, getSelectionMarks } from 'ngx-editor/helpers';
import { removeLink } from 'ngx-editor/commands';

const DEFAULT_PLACEHOLDER = 'Type Here...';
const PLACEHOLDER_CLASSNAME = 'NgxEditor__Placeholder';
const placeholderPlugin = (text = DEFAULT_PLACEHOLDER) => {
    return new Plugin({
        key: new PluginKey('placeholder'),
        props: {
            decorations(state) {
                var _a;
                const doc = state.doc;
                if (doc.childCount === 1 && ((_a = doc === null || doc === void 0 ? void 0 : doc.firstChild) === null || _a === void 0 ? void 0 : _a.isTextblock) && doc.firstChild.content.size === 0) {
                    const placeHolderEl = document.createElement('span');
                    placeHolderEl.classList.add(PLACEHOLDER_CLASSNAME);
                    placeHolderEl.textContent = text;
                    return DecorationSet.create(doc, [Decoration.widget(1, placeHolderEl)]);
                }
                return DecorationSet.empty;
            }
        }
    });
};
const ɵ0 = placeholderPlugin;

class LinkOptions {
    constructor(view) {
        this.bubbleEL = document.createElement('div');
        this.render(view);
        this.update(view);
    }
    render(view) {
        var _a;
        this.bubbleEL.className = 'NgxEditor__FloatingBubble';
        (_a = view.dom.parentNode) === null || _a === void 0 ? void 0 : _a.appendChild(this.bubbleEL);
    }
    setDomPosition(view) {
        // Otherwise, reposition it and update its content
        this.bubbleEL.style.display = '';
        const { bottom, left } = calculateBubblePos(view, this.bubbleEL);
        this.bubbleEL.style.left = left + 'px';
        this.bubbleEL.style.bottom = bottom + 'px';
    }
    createLinkNode(item, removeCB) {
        const el = document.createDocumentFragment();
        const link = document.createElement('a');
        link.href = item.attrs.href;
        link.target = '_blank';
        link.innerText = item.attrs.href;
        link.title = item.attrs.href;
        link.addEventListener('mousedown', (e) => {
            e.preventDefault();
            e.stopPropagation();
        }, { once: true });
        const commands = document.createElement('div');
        commands.classList.add('commands');
        const removeOpt = document.createElement('button');
        removeOpt.textContent = 'Remove Link';
        removeOpt.title = 'Remove link';
        removeOpt.classList.add('command');
        removeOpt.addEventListener('mousedown', removeCB, { once: true });
        commands.appendChild(removeOpt);
        el.appendChild(link);
        el.appendChild(commands);
        return el;
    }
    clearBubbleContent() {
        this.bubbleEL.textContent = '';
    }
    hideBubble() {
        this.bubbleEL.style.display = 'none';
    }
    update(view) {
        const { state, dispatch } = view;
        const { schema, selection } = state;
        if (!schema.marks.link) {
            return;
        }
        if (selection instanceof NodeSelection) {
            if (selection.node.type.name === 'image') {
                return;
            }
        }
        const hasFocus = view.hasFocus();
        const isActive = isLinkActive(state);
        const linkMarks = getSelectionMarks(state).filter(mark => mark.type === schema.marks.link);
        // hide for selection and show only for clicks
        if (!hasFocus || !isActive) {
            this.hideBubble();
            return;
        }
        const [linkItem] = linkMarks;
        this.clearBubbleContent();
        const removeCB = (e) => {
            e.preventDefault();
            e.stopPropagation();
            if (e.button !== 0) {
                return;
            }
            removeLink()(state, dispatch);
            view.focus();
        };
        const el = this.createLinkNode(linkItem, removeCB);
        this.bubbleEL.appendChild(el);
        // update dom position
        this.setDomPosition(view);
    }
    destroy() {
        this.bubbleEL.remove();
    }
}
const linkPlugin = () => {
    return new Plugin({
        key: new PluginKey('link'),
        view(editorView) {
            return new LinkOptions(editorView);
        },
        props: {
            handleDOMEvents: {
                blur: (view) => {
                    // dummy transaction to update
                    view.dispatch(view.state.tr.setMeta('LINK_PLUGIN_EDITOR_BLUR', true));
                    return false;
                }
            }
        }
    });
};
const ɵ0$1 = linkPlugin;

const WRAPPER_CLASSNAME = 'NgxEditor__ImageWrapper';
const WRAPPER_RESIZE_ACTIVE_CLASSNAME = 'NgxEditor__Resizer--Active';
const RESIZE_HANDLE_CLASSNAME = 'NgxEditor__ResizeHandle';
class ImageRezieView {
    constructor(node, view, getPos) {
        var _a, _b;
        const outer = document.createElement('span');
        outer.className = WRAPPER_CLASSNAME;
        outer.style.width = node.attrs.width;
        const handle = document.createElement('span');
        handle.className = RESIZE_HANDLE_CLASSNAME;
        const img = document.createElement('img');
        img.setAttribute('src', node.attrs.src);
        img.setAttribute('alt', (_a = node.attrs.alt) !== null && _a !== void 0 ? _a : '');
        img.setAttribute('title', (_b = node.attrs.title) !== null && _b !== void 0 ? _b : '');
        img.style.width = '100%';
        const handleBottomRight = document.createElement('span');
        const handleTopRight = document.createElement('span');
        const handleTopLeft = document.createElement('span');
        const handleBottomLeft = document.createElement('span');
        handleBottomRight.className = `${RESIZE_HANDLE_CLASSNAME}--BR`;
        handleTopRight.className = `${RESIZE_HANDLE_CLASSNAME}--TR`;
        handleTopLeft.className = `${RESIZE_HANDLE_CLASSNAME}--TL`;
        handleBottomLeft.className = `${RESIZE_HANDLE_CLASSNAME}--BL`;
        const resizePropoptionally = (evt) => {
            evt.preventDefault();
            const { state, dispatch } = view;
            const { tr } = state;
            const startX = evt.pageX;
            const startWidth = img.clientWidth;
            const { width } = window.getComputedStyle(view.dom);
            const editorWidth = parseInt(width, 10);
            const onMouseMove = (e) => {
                const currentX = e.pageX;
                const diffInPx = currentX - startX;
                const computedWidth = startWidth + diffInPx;
                // prevent image overflow the editor
                // prevent resizng below 20px
                if (computedWidth > editorWidth || computedWidth < 20) {
                    return;
                }
                outer.style.width = `${computedWidth}px`;
            };
            const onMouseUp = (e) => {
                e.preventDefault();
                document.removeEventListener('mousemove', onMouseMove);
                document.removeEventListener('mouseup', onMouseUp);
                const transaction = tr.setNodeMarkup(getPos(), null, {
                    src: node.attrs.src,
                    width: outer.style.width
                });
                const resolvedPos = transaction.doc.resolve(getPos());
                const newSelection = new NodeSelection(resolvedPos);
                transaction.setSelection(newSelection);
                dispatch(transaction);
            };
            document.addEventListener('mousemove', onMouseMove);
            document.addEventListener('mouseup', onMouseUp);
        };
        handleBottomRight.addEventListener('mousedown', resizePropoptionally, { once: true });
        handleTopRight.addEventListener('mousedown', resizePropoptionally, { once: true });
        handleTopLeft.addEventListener('mousedown', resizePropoptionally, { once: true });
        handleBottomLeft.addEventListener('mousedown', resizePropoptionally, { once: true });
        handle.appendChild(handleBottomRight);
        handle.appendChild(handleTopRight);
        handle.appendChild(handleTopLeft);
        handle.appendChild(handleBottomLeft);
        outer.appendChild(handle);
        outer.appendChild(img);
        this.dom = outer;
        this.img = img;
        this.handle = handle;
    }
    selectNode() {
        this.dom.classList.add(WRAPPER_RESIZE_ACTIVE_CLASSNAME);
        this.handle.style.display = 'block';
    }
    deselectNode() {
        this.dom.classList.remove(WRAPPER_RESIZE_ACTIVE_CLASSNAME);
        this.handle.style.display = 'none';
    }
}
const defaultOptions = {
    resize: true,
};
const imagePlugin = (opts = defaultOptions) => {
    const options = Object.assign(Object.assign({}, defaultOptions), opts);
    return new Plugin({
        key: new PluginKey('link'),
        props: {
            nodeViews: {
                image: (node, view, getPos) => {
                    if (!options.resize) {
                        return null;
                    }
                    return new ImageRezieView(node, view, getPos);
                },
            }
        }
    });
};
const ɵ0$2 = imagePlugin;

/**
 * Generated bundle index. Do not edit.
 */

export { imagePlugin, linkPlugin as link, placeholderPlugin as placeholder };
//# sourceMappingURL=ngx-editor-plugins.js.map
